Index: include/speex/speex.h
===================================================================
--- include/speex/speex.h	(revision 10524)
+++ include/speex/speex.h	(working copy)
@@ -374,6 +374,8 @@
  */
 int speex_decoder_ctl(void *state, int request, void *ptr);
 
+/* set assembler pointers for windows/darwin- takes encoder or decoder state */
+void speex_set_asm(int use_asm);
 
 /** Query function for mode information
  *
Index: libspeex/arch.h
===================================================================
--- libspeex/arch.h	(revision 10524)
+++ libspeex/arch.h	(working copy)
@@ -79,9 +79,7 @@
 #ifdef FIXED_DEBUG
 #include "fixed_debug.h"
 #else
-
 #include "fixed_generic.h"
-
 #ifdef ARM5E_ASM
 #include "fixed_arm5e.h"
 #elif defined (ARM4_ASM)
@@ -91,12 +89,11 @@
 #elif defined (BFIN_ASM)
 #include "fixed_bfin.h"
 #endif
-
 #endif
 
-
 #else
 
+
 typedef float spx_mem_t;
 typedef float spx_coef_t;
 typedef float spx_lsp_t;
@@ -170,7 +167,6 @@
 #define DIV32_16(a,b)     ((a)/(b))
 #define DIV32(a,b)     ((a)/(b))
 
-
 #endif
 
 
@@ -189,4 +185,30 @@
 
 #endif
 
+// way to make sse vector types more portable
+#if defined _USE_SSE
+#include <xmmintrin.h>
+#endif
+
+
+typedef union __spx_vec16_t
+{
+#if defined _USE_SSE
+	__m128 m;
+	float f[4];
+#else
+	spx_word16_t f[1];
+#endif
+} spx_vec16_t;
+
+typedef union __spx_vec32_t
+{
+#if defined _USE_SSE
+	__m128 m;
+	float f[4];
+#else
+	spx_word32_t f[1];
+#endif
+} spx_vec32_t;
+
 #endif
Index: libspeex/cb_search.c
===================================================================
--- libspeex/cb_search.c	(revision 10524)
+++ libspeex/cb_search.c	(working copy)
@@ -34,33 +34,27 @@
 #endif
 
 #include "cb_search.h"
-#include "filters.h"
 #include "stack_alloc.h"
 #include "vq.h"
 #include "misc.h"
+#include "filters.h"
 
-#ifdef _USE_SSE
-#include "cb_search_sse.h"
-#elif defined(ARM4_ASM) || defined(ARM5E_ASM)
-#include "cb_search_arm4.h"
-#elif defined(BFIN_ASM)
-#include "cb_search_bfin.h"
-#endif
-
 #ifndef OVERRIDE_COMPUTE_WEIGHTED_CODEBOOK
-static void compute_weighted_codebook(const signed char *shape_cb, const spx_word16_t *r, spx_word16_t *resp, spx_word16_t *resp2, spx_word32_t *E, int shape_cb_size, int subvect_size, char *stack)
+void compute_weighted_codebook_c(const signed char *shape_cb, const spx_word16_t *r, spx_word16_t *resp, spx_vec16_t *resp2, spx_vec32_t *E, int shape_cb_size, int subvect_size, char *stack)
 {
    int i, j, k;
    VARDECL(spx_word16_t *shape);
    ALLOC(shape, subvect_size, spx_word16_t);
    for (i=0;i<shape_cb_size;i++)
    {
-      spx_word16_t *res;
-      
-      res = resp+i*subvect_size;
+      spx_word16_t *res = resp+i*subvect_size;
+      spx_vec16_t *res2 = resp2+i*subvect_size;
+
       for (k=0;k<subvect_size;k++)
          shape[k] = (spx_word16_t)shape_cb[i*subvect_size+k];
-      E[i]=0;
+      
+	  E[i].f[0]=0;
+	  //E[i] = 0;
 
       /* Compute codeword response using convolution with impulse response */
       for(j=0;j<subvect_size;j++)
@@ -75,8 +69,11 @@
          res16 = 0.03125f*resj;
 #endif
          /* Compute codeword energy */
-         E[i]=MAC16_16(E[i],res16,res16);
-         res[j] = res16;
+         E[i].f[0]=MAC16_16(E[i].f[0],res16,res16);
+         //E[i]=MAC16_16(E[i],res16,res16);
+
+		 res[j] = res16;
+		 res2[j].f[0] = res16;
          /*printf ("%d\n", (int)res[j]);*/
       }
    }
@@ -84,6 +81,7 @@
 }
 #endif
 
+
 #ifndef OVERRIDE_TARGET_UPDATE
 static inline void target_update(spx_word16_t *t, spx_word16_t g, spx_word16_t *r, int len)
 {
@@ -117,13 +115,18 @@
    int n;
 #endif
    VARDECL(spx_word16_t *resp);
-#ifdef _USE_SSE
-   VARDECL(__m128 *resp2);
-   VARDECL(__m128 *E);
-#else
-   spx_word16_t *resp2;
-   VARDECL(spx_word32_t *E);
+/*
+#ifdef _USE_SSE
+   VARDECL(__m128 *resp2);
+   VARDECL(__m128 *E);
+#else
+   spx_word16_t *resp2;
+   VARDECL(spx_word32_t *E);
 #endif
+*/
+   VARDECL(spx_vec16_t *resp2); // not used in the c version
+   VARDECL(spx_vec32_t *E);
+
    VARDECL(spx_word16_t *t);
    VARDECL(spx_sig_t *e);
    const signed char *shape_cb;
@@ -146,13 +149,19 @@
    shape_cb = params->shape_cb;
    have_sign = params->have_sign;
    ALLOC(resp, shape_cb_size*subvect_size, spx_word16_t);
-#ifdef _USE_SSE
-   ALLOC(resp2, (shape_cb_size*subvect_size)>>2, __m128);
-   ALLOC(E, shape_cb_size>>2, __m128);
-#else
-   resp2 = resp;
-   ALLOC(E, shape_cb_size, spx_word32_t);
+
+/*
+#ifdef _USE_SSE
+   ALLOC(resp2, (shape_cb_size*subvect_size)>>2, __m128);
+   ALLOC(E, shape_cb_size>>2, __m128);
+#else
+   resp2 = resp;
+   ALLOC(E, shape_cb_size, spx_word32_t);
 #endif
+*/
+   ALLOC(resp2, shape_cb_size*subvect_size, spx_vec16_t);
+   ALLOC(E, shape_cb_size, spx_vec32_t);
+
    ALLOC(t, nsf, spx_word16_t);
    ALLOC(e, nsf, spx_sig_t);
    
@@ -170,7 +179,7 @@
          vq_nbest_sign(x, resp2, subvect_size, shape_cb_size, E, 1, &best_index, &best_dist, stack);
       else
          vq_nbest(x, resp2, subvect_size, shape_cb_size, E, 1, &best_index, &best_dist, stack);
-      
+
       speex_bits_pack(bits,best_index,params->shape_bits+have_sign);
       
       {
@@ -268,13 +277,18 @@
 {
    int i,j,k,m,n,q;
    VARDECL(spx_word16_t *resp);
-#ifdef _USE_SSE
-   VARDECL(__m128 *resp2);
-   VARDECL(__m128 *E);
-#else
-   spx_word16_t *resp2;
-   VARDECL(spx_word32_t *E);
+/*
+#ifdef _USE_SSE
+   VARDECL(__m128 *resp2);
+   VARDECL(__m128 *E);
+#else
+   spx_word16_t *resp2;
+   VARDECL(spx_word32_t *E);
 #endif
+*/
+   VARDECL(spx_vec16_t *resp2);
+   VARDECL(spx_vec32_t *E);
+   
    VARDECL(spx_word16_t *t);
    VARDECL(spx_sig_t *e);
    VARDECL(spx_sig_t *r2);
@@ -320,13 +334,19 @@
    shape_cb = params->shape_cb;
    have_sign = params->have_sign;
    ALLOC(resp, shape_cb_size*subvect_size, spx_word16_t);
-#ifdef _USE_SSE
-   ALLOC(resp2, (shape_cb_size*subvect_size)>>2, __m128);
-   ALLOC(E, shape_cb_size>>2, __m128);
-#else
-   resp2 = resp;
-   ALLOC(E, shape_cb_size, spx_word32_t);
+
+/*
+#ifdef _USE_SSE
+   ALLOC(resp2, (shape_cb_size*subvect_size)>>2, __m128);
+   ALLOC(E, shape_cb_size>>2, __m128);
+#else
+   resp2 = resp;
+   ALLOC(E, shape_cb_size, spx_word32_t);
 #endif
+*/
+   ALLOC(resp2, shape_cb_size*subvect_size, spx_vec16_t);
+   ALLOC(E, shape_cb_size, spx_vec32_t);
+
    ALLOC(t, nsf, spx_word16_t);
    ALLOC(e, nsf, spx_sig_t);
    ALLOC(r2, nsf, spx_sig_t);
@@ -366,7 +386,7 @@
 
    for (j=0;j<N;j++)
       odist[j]=0;
-   
+
    /*For all subvectors*/
    for (i=0;i<nb_subvect;i++)
    {
@@ -552,7 +572,10 @@
          signs[i] = speex_bits_unpack_unsigned(bits, 1);
       else
          signs[i] = 0;
+
+	  /* if abs(ind[i] is > 256 it is likely stack corruption */
       ind[i] = speex_bits_unpack_unsigned(bits, params->shape_bits);
+
    }
    /* Compute decoded excitation */
    for (i=0;i<nb_subvect;i++)
Index: libspeex/cb_search_sse.h
===================================================================
--- libspeex/cb_search_sse.h	(revision 10524)
+++ libspeex/cb_search_sse.h	(working copy)
@@ -32,53 +32,14 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#ifndef _CBSEARCH_SSE_
+#define _CBSEARCH_SSE_
+
 #include <xmmintrin.h>
+#include <speex/speex_bits.h>
+#include "stack_alloc.h"
+#include "misc.h"
 
-static inline void _spx_mm_getr_ps (__m128 U, float *__Z, float *__Y, float *__X, float *__W)
-{
-  union {
-    float __a[4];
-    __m128 __v;
-  } __u;
-  
-  __u.__v = U;
+void compute_weighted_codebook_sse(const signed char *shape_cb, const spx_word16_t *_r, spx_word16_t *resp, spx_vec16_t *resp2, spx_vec32_t *E, int shape_cb_size, int subvect_size, char *stack);
 
-  *__Z = __u.__a[0];
-  *__Y = __u.__a[1];
-  *__X = __u.__a[2];
-  *__W = __u.__a[3];
-
-}
-
-#define OVERRIDE_COMPUTE_WEIGHTED_CODEBOOK
-static void compute_weighted_codebook(const signed char *shape_cb, const spx_sig_t *_r, float *resp, __m128 *resp2, __m128 *E, int shape_cb_size, int subvect_size, char *stack)
-{
-   int i, j, k;
-   __m128 resj, EE;
-   VARDECL(__m128 *r);
-   VARDECL(__m128 *shape);
-   ALLOC(r, subvect_size, __m128);
-   ALLOC(shape, subvect_size, __m128);
-   for(j=0;j<subvect_size;j++)
-      r[j] = _mm_load_ps1(_r+j);
-   for (i=0;i<shape_cb_size;i+=4)
-   {
-      float *_res = resp+i*subvect_size;
-      const signed char *_shape = shape_cb+i*subvect_size;
-      EE = _mm_setzero_ps();
-      for(j=0;j<subvect_size;j++)
-      {
-         shape[j] = _mm_setr_ps(0.03125*_shape[j], 0.03125*_shape[subvect_size+j], 0.03125*_shape[2*subvect_size+j], 0.03125*_shape[3*subvect_size+j]);
-      }
-      for(j=0;j<subvect_size;j++)
-      {
-         resj = _mm_setzero_ps();
-         for (k=0;k<=j;k++)
-            resj = _mm_add_ps(resj, _mm_mul_ps(shape[k],r[j-k]));
-         _spx_mm_getr_ps(resj, _res+j, _res+subvect_size+j, _res+2*subvect_size+j, _res+3*subvect_size+j);
-         *resp2++ = resj;
-         EE = _mm_add_ps(EE, _mm_mul_ps(resj, resj));
-      }
-      E[i>>2] = EE;
-   }
-}
+#endif
Index: libspeex/fftwrap.c
===================================================================
--- libspeex/fftwrap.c	(revision 10524)
+++ libspeex/fftwrap.c	(working copy)
@@ -62,8 +62,8 @@
    if (in==out)
    {
       int i;
-      speex_warning("FFT should not be done in-place");
-      float scale = 1./((struct drft_lookup *)table)->n;
+      float scale = 1.f/((struct drft_lookup *)table)->n;
+//      speex_warning("FFT should not be done in-place");
       for (i=0;i<((struct drft_lookup *)table)->n;i++)
          out[i] = scale*in[i];
    } else {
Index: libspeex/filters.c
===================================================================
--- libspeex/filters.c	(revision 10524)
+++ libspeex/filters.c	(working copy)
@@ -41,16 +41,7 @@
 #include "ltp.h"
 #include <math.h>
 
-#ifdef _USE_SSE
-#include "filters_sse.h"
-#elif defined (ARM4_ASM) || defined(ARM5E_ASM)
-#include "filters_arm4.h"
-#elif defined (BFIN_ASM)
-#include "filters_bfin.h"
-#endif
 
-
-
 void bw_lpc(spx_word16_t gamma, const spx_coef_t *lpc_in, spx_coef_t *lpc_out, int order)
 {
    int i;
@@ -211,8 +202,8 @@
 
 
 #ifndef OVERRIDE_FILTER_MEM2
+void filter_mem2_c(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 #ifdef PRECISION16
-void filter_mem2(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_word16_t xi,yi,nyi;
@@ -231,7 +222,6 @@
    }
 }
 #else
-void filter_mem2(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_sig_t xi,yi,nyi;
@@ -252,9 +242,10 @@
 #endif
 #endif
 
+
 #ifndef OVERRIDE_IIR_MEM2
+void iir_mem2_c(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 #ifdef PRECISION16
-void iir_mem2(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_word16_t yi,nyi;
@@ -272,7 +263,6 @@
    }
 }
 #else
-void iir_mem2(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_word32_t xi,yi,nyi;
@@ -294,8 +284,8 @@
 #endif
 
 #ifndef OVERRIDE_FIR_MEM2
+void fir_mem2_c(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 #ifdef PRECISION16
-void fir_mem2(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_word16_t xi,yi;
@@ -313,7 +303,6 @@
    }
 }
 #else
-void fir_mem2(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_word32_t xi,yi;
@@ -333,13 +322,6 @@
 #endif
 #endif
 
-
-
-
-
-
-
-
 void syn_percep_zero(const spx_sig_t *xx, const spx_coef_t *ak, const spx_coef_t *awk1, const spx_coef_t *awk2, spx_sig_t *y, int N, int ord, char *stack)
 {
    int i;
@@ -493,6 +475,7 @@
       mem[i+1] = xx[i];
 }
 
+
 void filter_dc_notch(spx_sig_t *in, spx_word16_t radius, spx_sig_t *out, int len, spx_mem_t *mem)
 {
    int i;
@@ -619,4 +602,4 @@
       new_exc[i] *= mem->smooth_gain;
    }
 #endif
-}
+}
\ No newline at end of file
Index: libspeex/filters.h
===================================================================
--- libspeex/filters.h	(revision 10524)
+++ libspeex/filters.h	(working copy)
@@ -32,21 +32,11 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef FILTERS_H
-#define FILTERS_H
+#ifndef _FILTERS_H_
+#define _FILTERS_H_
 
 #include "misc.h"
 
-spx_word16_t compute_rms(const spx_sig_t *x, int len);
-void signal_mul(const spx_sig_t *x, spx_sig_t *y, spx_word32_t scale, int len);
-void signal_div(const spx_sig_t *x, spx_sig_t *y, spx_word32_t scale, int len);
-
-#ifdef FIXED_POINT
-
-int normalize16(const spx_sig_t *x, spx_word16_t *y, spx_sig_t max_scale, int len);
-
-#endif
-
 /** Combined filter memory. */
 typedef struct {
    int   last_pitch;
@@ -54,26 +44,6 @@
    spx_word16_t smooth_gain;
 } CombFilterMem;
 
-
-void qmf_decomp(const spx_word16_t *xx, const spx_word16_t *aa, spx_sig_t *, spx_sig_t *y2, int N, int M, spx_word16_t *mem, char *stack);
-void fir_mem_up(const spx_sig_t *x, const spx_word16_t *a, spx_sig_t *y, int N, int M, spx_word32_t *mem, char *stack);
-
-
-void filter_mem2(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
-void fir_mem2(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
-void iir_mem2(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
-
-/* Apply bandwidth expansion on LPC coef */
-void bw_lpc(spx_word16_t , const spx_coef_t *lpc_in, spx_coef_t *lpc_out, int order);
-
-
-
-void syn_percep_zero(const spx_sig_t *x, const spx_coef_t *ak, const spx_coef_t *awk1, const spx_coef_t *awk2, spx_sig_t *y, int N, int ord, char *stack);
-
-void residue_percep_zero(const spx_sig_t *xx, const spx_coef_t *ak, const spx_coef_t *awk1, const spx_coef_t *awk2, spx_sig_t *y, int N, int ord, char *stack);
-
-void compute_impulse_response(const spx_coef_t *ak, const spx_coef_t *awk1, const spx_coef_t *awk2, spx_word16_t *y, int N, int ord, char *stack);
-
 void comb_filter_mem_init (CombFilterMem *mem);
 
 void comb_filter(
@@ -88,5 +58,91 @@
 CombFilterMem *mem
 );
 
+spx_word16_t compute_rms(const spx_sig_t *x, int len);
+void signal_mul(const spx_sig_t *x, spx_sig_t *y, spx_word32_t scale, int len);
+void signal_div(const spx_sig_t *x, spx_sig_t *y, spx_word32_t scale, int len);
 
+#ifdef FIXED_POINT
+int normalize16(const spx_sig_t *x, spx_word16_t *y, spx_sig_t max_scale, int len);
 #endif
+
+void qmf_decomp(const spx_word16_t *xx, const spx_word16_t *aa, spx_sig_t *, spx_sig_t *y2, int N, int M, spx_word16_t *mem, char *stack);
+void fir_mem_up(const spx_sig_t *x, const spx_word16_t *a, spx_sig_t *y, int N, int M, spx_word32_t *mem, char *stack);
+
+/* Apply bandwidth expansion on LPC coef */
+void bw_lpc(spx_word16_t , const spx_coef_t *lpc_in, spx_coef_t *lpc_out, int order);
+void residue_percep_zero(const spx_sig_t *xx, const spx_coef_t *ak, const spx_coef_t *awk1, const spx_coef_t *awk2, spx_sig_t *y, int N, int ord, char *stack);
+void compute_impulse_response(const spx_coef_t *ak, const spx_coef_t *awk1, const spx_coef_t *awk2, spx_word16_t *y, int N, int ord, char *stack);
+void syn_percep_zero(const spx_sig_t *xx, const spx_coef_t *ak, const spx_coef_t *awk1, const spx_coef_t *awk2, spx_sig_t *y, int N, int ord, char *stack);
+
+#if defined (_USE_SSE)
+
+    typedef spx_word32_t (*inner_prod_func)(const float *, const float *, int);
+    typedef void (*pitch_xcorr_func)(const float *, const float *, float *, int, int, char *);
+    typedef void (*filter_mem2_func)(const spx_sig_t *, const spx_coef_t *, const spx_coef_t *, spx_sig_t *, int, int, spx_mem_t *);
+    typedef void (*iir_mem2_func)(const spx_sig_t *, const spx_coef_t *, spx_sig_t *, int, int, spx_mem_t *);
+    typedef void (*fir_mem2_func)(const spx_sig_t *, const spx_coef_t *, spx_sig_t *, int, int, spx_mem_t *);
+    typedef void (*vq_nbest_func)(spx_word16_t *_in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
+    typedef void (*vq_nbest_sign_func)(spx_word16_t *_in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
+    typedef void (*compute_weighted_codebook_func)(const signed char *shape_cb, const spx_word16_t *_r, spx_word16_t *resp, spx_vec16_t *resp2, spx_vec32_t *E, int shape_cb_size, int subvect_size, char *stack);
+
+    filter_mem2_func fptr_fm;
+    iir_mem2_func fptr_iir;
+    fir_mem2_func fptr_fir;
+    inner_prod_func fptr_ip;
+    pitch_xcorr_func fptr_px;
+    vq_nbest_func fptr_vnb;
+    vq_nbest_sign_func fptr_vnbs;
+    compute_weighted_codebook_func fptr_cwc;
+
+    #define filter_mem2 (*fptr_fm)
+    #define iir_mem2 (*fptr_iir)
+    #define fir_mem2 (*fptr_fir)
+    #define inner_prod (*fptr_ip)
+    #define pitch_xcorr (*fptr_px)
+    #define vq_nbest (*fptr_vnb)
+    #define vq_nbest_sign (*fptr_vnbs)
+    #define compute_weighted_codebook (*fptr_cwc)
+
+#elif defined (_USE_ALTIVEC)
+
+    typedef void (*filter_mem2_func)(const spx_sig_t *, const spx_coef_t *, const spx_coef_t *, spx_sig_t *, int, int, spx_mem_t *);
+    typedef spx_word32_t (*inner_prod_func)(const float *, const float *, int);
+
+    filter_mem2_func fptr_fm;
+    inner_prod_func fptr_ip;
+
+    #define filter_mem2 (*fptr_fm)
+    #define inner_prod (*fptr_ip)
+    #define fir_mem2 fir_mem2_c
+    #define iir_mem2 iir_mem2_c
+    #define vq_nbest_sign vq_nbest_sign_c
+    #define vq_nbest vq_nbest_c
+    #define pitch_xcorr pitch_xcorr_c
+    #define compute_weighted_codebook compute_weighted_codebook_c
+
+#else
+
+    // may not get used due to overrides
+    #define filter_mem2 filter_mem2_c
+    #define inner_prod inner_prod_c
+    #define fir_mem2 fir_mem2_c
+    #define iir_mem2 iir_mem2_c
+    #define vq_nbest_sign vq_nbest_sign_c
+    #define vq_nbest vq_nbest_c
+    #define pitch_xcorr pitch_xcorr_c
+    #define compute_weighted_codebook compute_weighted_codebook_c
+
+#endif 
+
+void filter_mem2_c(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+void fir_mem2_c(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+void iir_mem2_c(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+spx_word32_t inner_prod_c(const spx_word16_t *x, const spx_word16_t *y, int len);
+void pitch_xcorr_c(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack);
+void vq_nbest_sign_c(spx_word16_t *in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
+void vq_nbest_c(spx_word16_t *in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
+void compute_weighted_codebook_c(const signed char *shape_cb, const spx_word16_t *_r, spx_word16_t *resp, spx_vec16_t *resp2, spx_vec32_t *E, int shape_cb_size, int subvect_size, char *stack);
+
+
+#endif
Index: libspeex/filters_sse.h
===================================================================
--- libspeex/filters_sse.h	(revision 10524)
+++ libspeex/filters_sse.h	(working copy)
@@ -32,305 +32,22 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <xmmintrin.h>
+#ifndef _FILTERS_SSE_
+#define _FILTERS_SSE_
 
-void filter_mem2_10(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   __m128 num[3], den[3], mem[3];
+#include "misc.h"
 
-   int i;
+void filter_mem2_10(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem);
+void filter_mem2_8(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem);
 
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      num[i] = _mm_loadu_ps(_num+4*i);
-      den[i] = _mm_loadu_ps(_den+4*i);
-   }
-   mem[2] = _mm_setr_ps(_mem[8], _mem[9], 0, 0);
-   num[2] = _mm_setr_ps(_num[8], _num[9], 0, 0);
-   den[2] = _mm_setr_ps(_den[8], _den[9], 0, 0);
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
+void iir_mem2_10(const float *x, const float *_den, float *y, int N, int ord, float *_mem);
+void iir_mem2_8(const float *x, const float *_den, float *y, int N, int ord, float *_mem);
 
-      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
-      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
+void fir_mem2_10(const float *x, const float *_num, float *y, int N, int ord, float *_mem);
+void fir_mem2_8(const float *x, const float *_num, float *y, int N, int ord, float *_mem);
 
-      mem[1] = _mm_move_ss(mem[1], mem[2]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
+void filter_mem2_sse(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+void iir_mem2_sse(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+void fir_mem2_sse(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
 
-      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
-      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
-
-      mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0xfd);
-
-      mem[2] = _mm_add_ps(mem[2], _mm_mul_ps(xx, num[2]));
-      mem[2] = _mm_sub_ps(mem[2], _mm_mul_ps(yy, den[2]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-   _mm_store_ss(_mem+8, mem[2]);
-   mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0x55);
-   _mm_store_ss(_mem+9, mem[2]);
-}
-
-void filter_mem2_8(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   __m128 num[2], den[2], mem[2];
-
-   int i;
-
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      num[i] = _mm_loadu_ps(_num+4*i);
-      den[i] = _mm_loadu_ps(_den+4*i);
-   }
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
-
-      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
-      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
-
-      mem[1] = _mm_sub_ss(mem[1], mem[1]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
-
-      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
-      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-}
-
-
-#define OVERRIDE_FILTER_MEM2
-void filter_mem2(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   if(ord==10)
-      filter_mem2_10(x, _num, _den, y, N, ord, _mem);
-   else if (ord==8)
-      filter_mem2_8(x, _num, _den, y, N, ord, _mem);
-}
-
-
-
-void iir_mem2_10(const float *x, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   __m128 den[3], mem[3];
-
-   int i;
-
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      den[i] = _mm_loadu_ps(_den+4*i);
-   }
-   mem[2] = _mm_setr_ps(_mem[8], _mem[9], 0, 0);
-   den[2] = _mm_setr_ps(_den[8], _den[9], 0, 0);
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
-
-      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
-
-      mem[1] = _mm_move_ss(mem[1], mem[2]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
-
-      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
-
-      mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0xfd);
-
-      mem[2] = _mm_sub_ps(mem[2], _mm_mul_ps(yy, den[2]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-   _mm_store_ss(_mem+8, mem[2]);
-   mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0x55);
-   _mm_store_ss(_mem+9, mem[2]);
-}
-
-
-void iir_mem2_8(const float *x, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   __m128 den[2], mem[2];
-
-   int i;
-
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      den[i] = _mm_loadu_ps(_den+4*i);
-   }
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
-
-      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
-
-      mem[1] = _mm_sub_ss(mem[1], mem[1]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
-
-      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-}
-
-#define OVERRIDE_IIR_MEM2
-void iir_mem2(const float *x, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   if(ord==10)
-      iir_mem2_10(x, _den, y, N, ord, _mem);
-   else if (ord==8)
-      iir_mem2_8(x, _den, y, N, ord, _mem);
-}
-
-
-void fir_mem2_10(const float *x, const float *_num, float *y, int N, int ord, float *_mem)
-{
-   __m128 num[3], mem[3];
-
-   int i;
-
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      num[i] = _mm_loadu_ps(_num+4*i);
-   }
-   mem[2] = _mm_setr_ps(_mem[8], _mem[9], 0, 0);
-   num[2] = _mm_setr_ps(_num[8], _num[9], 0, 0);
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
-
-      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
-
-      mem[1] = _mm_move_ss(mem[1], mem[2]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
-
-      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
-
-      mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0xfd);
-
-      mem[2] = _mm_add_ps(mem[2], _mm_mul_ps(xx, num[2]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-   _mm_store_ss(_mem+8, mem[2]);
-   mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0x55);
-   _mm_store_ss(_mem+9, mem[2]);
-}
-
-void fir_mem2_8(const float *x, const float *_num, float *y, int N, int ord, float *_mem)
-{
-   __m128 num[2], mem[2];
-
-   int i;
-
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      num[i] = _mm_loadu_ps(_num+4*i);
-   }
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
-
-      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
-
-      mem[1] = _mm_sub_ss(mem[1], mem[1]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
-
-      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-}
-
-#define OVERRIDE_FIR_MEM2
-void fir_mem2(const float *x, const float *_num, float *y, int N, int ord, float *_mem)
-{
-   if(ord==10)
-      fir_mem2_10(x, _num, y, N, ord, _mem);
-   else if (ord==8)
-      fir_mem2_8(x, _num, y, N, ord, _mem);
-}
+#endif
\ No newline at end of file
Index: libspeex/ltp.c
===================================================================
--- libspeex/ltp.c	(revision 10524)
+++ libspeex/ltp.c	(working copy)
@@ -45,17 +45,12 @@
 #define NULL 0
 #endif
 
-
-#ifdef _USE_SSE
-#include "ltp_sse.h"
-#elif defined (ARM4_ASM) || defined(ARM5E_ASM)
-#include "ltp_arm4.h"
-#elif defined (BFIN_ASM)
-#include "ltp_bfin.h"
-#endif
-
 #ifndef OVERRIDE_INNER_PROD
-static spx_word32_t inner_prod(const spx_word16_t *x, const spx_word16_t *y, int len)
+#if defined _USE_SSE || defined _USE_ALTIVEC
+spx_word32_t inner_prod_c(const spx_word16_t *x, const spx_word16_t *y, int len)
+#else
+static inline spx_word32_t inner_prod_c(const spx_word16_t *x, const spx_word16_t *y, int len)
+#endif
 {
    spx_word32_t sum=0;
    len >>= 2;
@@ -75,7 +70,7 @@
 
 #ifndef OVERRIDE_PITCH_XCORR
 #if 0 /* HINT: Enable this for machines with enough registers (i.e. not x86) */
-static void pitch_xcorr(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack)
+void pitch_xcorr_c(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack)
 {
    int i,j;
    for (i=0;i<nb_pitch;i+=4)
@@ -138,7 +133,7 @@
 
 }
 #else
-static void pitch_xcorr(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack)
+void pitch_xcorr_c(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack)
 {
    int i;
    for (i=0;i<nb_pitch;i++)
@@ -176,7 +171,9 @@
    VARDECL(spx_word32_t *corr);
    VARDECL(spx_word32_t *energy);
    VARDECL(spx_word32_t *score);
+#ifdef FIXED_POINT
    VARDECL(spx_word16_t *swn2);
+#endif
    spx_word16_t *swn;
 
    ALLOC(best_score, N, spx_word32_t);
@@ -276,15 +273,15 @@
    /* Compute open-loop gain */
    if (gain)
    {
+       spx_word16_t g;
        for (j=0;j<N;j++)
        {
-          spx_word16_t g;
           i=pitch[j];
-          g = DIV32(corr[i-start], 10+SHR32(MULT16_16(spx_sqrt(e0),spx_sqrt(energy[i-start])),6));
+          //if things go wonky, enable this:if (energy[i-start] < -10) energy[i-start] = -10;
+          g = DIV32(corr[i-start], SHR32(MULT16_16(spx_sqrt(10 + e0), spx_sqrt(10+energy[i-start])),6));
           /* FIXME: g = max(g,corr/energy) */
-                   if (g<0)
-                   g = 0;
-             gain[j]=g;
+          if (g<0) g = 0;
+          gain[j]=g;
        }
    }
 }
Index: libspeex/ltp_sse.h
===================================================================
--- libspeex/ltp_sse.h	(revision 10524)
+++ libspeex/ltp_sse.h	(working copy)
@@ -31,62 +31,13 @@
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
+#ifndef _LTP_SSE_
+#define _LTP_SSE_
 
 #include <xmmintrin.h>
+#include "misc.h"
 
-#define OVERRIDE_INNER_PROD
-static float inner_prod(const float *a, const float *b, int len)
-{
-   int i;
-   float ret;
-   __m128 sum = _mm_setzero_ps();
-   for (i=0;i<(len>>2);i+=2)
-   {
-      sum = _mm_add_ps(sum, _mm_mul_ps(_mm_loadu_ps(a+0), _mm_loadu_ps(b+0)));
-      sum = _mm_add_ps(sum, _mm_mul_ps(_mm_loadu_ps(a+4), _mm_loadu_ps(b+4)));
-      a += 8;
-      b += 8;
-   }
-   sum = _mm_add_ps(sum, _mm_movehl_ps(sum, sum));
-   sum = _mm_add_ss(sum, _mm_shuffle_ps(sum, sum, 0x55));
-   _mm_store_ss(&ret, sum);
-   return ret;
-}
+spx_word32_t inner_prod_sse(const float *a, const float *b, int len);
+void pitch_xcorr_sse(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack);
 
-#define OVERRIDE_PITCH_XCORR
-static void pitch_xcorr(const float *_x, const float *_y, float *corr, int len, int nb_pitch, char *stack)
-{
-   int i, offset;
-   VARDECL(__m128 *x);
-   VARDECL(__m128 *y);
-   int N, L;
-   N = len>>2;
-   L = nb_pitch>>2;
-   ALLOC(x, N, __m128);
-   ALLOC(y, N+L, __m128);
-   for (i=0;i<N;i++)
-      x[i] = _mm_loadu_ps(_x+(i<<2));
-   for (offset=0;offset<4;offset++)
-   {
-      for (i=0;i<N+L;i++)
-         y[i] = _mm_loadu_ps(_y+(i<<2)+offset);
-      for (i=0;i<L;i++)
-      {
-         int j;
-         __m128 sum, *xx, *yy;
-         sum = _mm_setzero_ps();
-         yy = y+i;
-         xx = x;
-         for (j=0;j<N;j+=2)
-         {
-            sum = _mm_add_ps(sum, _mm_mul_ps(xx[0], yy[0]));
-            sum = _mm_add_ps(sum, _mm_mul_ps(xx[1], yy[1]));
-            xx += 2;
-            yy += 2;
-         }
-         sum = _mm_add_ps(sum, _mm_movehl_ps(sum, sum));
-         sum = _mm_add_ss(sum, _mm_shuffle_ps(sum, sum, 0x55));
-         _mm_store_ss(corr+nb_pitch-1-(i<<2)-offset, sum);
-      }
-   }
-}
+#endif
\ No newline at end of file
Index: libspeex/nb_celp.c
===================================================================
--- libspeex/nb_celp.c	(revision 10524)
+++ libspeex/nb_celp.c	(working copy)
@@ -63,14 +63,28 @@
 #define SUBMODE(x) st->submodes[st->submodeID]->x
 
 /* Default size for the encoder and decoder stack (can be changed at compile time).
-   This does not apply when using variable-size arrays or alloca. */
+   This does not apply when using variable-size arrays or alloca. 
+  
+  this was increased because of the SSE run time change- the reasoning is as follows
+  for wideband/narrowband, assuming the worst case is 1280 = shape_cb_size*subvect_size,
+  we are adding 1280 to the stack for resp2 for non-sse code, and 640*sizeof(__m128)/sizeof(float) 
+  or 640*4 = 2560 for the sse code.  Then we round up a bit to be safe. 
+*/
 #ifndef NB_ENC_STACK
-#define NB_ENC_STACK (8000*sizeof(spx_sig_t))
+#ifdef _USE_SSE
+#define NB_ENC_STACK (12000*sizeof(spx_sig_t))
+#else
+#define NB_ENC_STACK (9280*sizeof(spx_sig_t))
 #endif
+#endif
 
 #ifndef NB_DEC_STACK
-#define NB_DEC_STACK (4000*sizeof(spx_sig_t))
+#ifdef _USE_SSE
+#define NB_DEC_STACK (8000*sizeof(spx_sig_t))
+#else
+#define NB_DEC_STACK (5280*sizeof(spx_sig_t))
 #endif
+#endif
 
 
 #ifdef FIXED_POINT
@@ -372,6 +386,7 @@
                                nol_pitch, nol_pitch_coef, 6, stack);
          ol_pitch=nol_pitch[0];
          ol_pitch_coef = nol_pitch_coef[0];
+
          /*Try to remove pitch multiples*/
          for (i=1;i<6;i++)
          {
@@ -783,7 +798,8 @@
       for (i=0;i<st->lpcSize;i++)
          mem[i]=st->mem_sw[i];
       filter_mem2(exc, st->bw_lpc1, st->bw_lpc2, res, response_bound, st->lpcSize, mem);
-      for (i=response_bound;i<st->subframeSize;i++)
+
+for (i=response_bound;i<st->subframeSize;i++)
          res[i]=0;
 #else
       iir_mem2(exc, st->interp_qlpc, exc, st->subframeSize, st->lpcSize, mem);
Index: libspeex/sb_celp.c
===================================================================
--- libspeex/sb_celp.c	(revision 10524)
+++ libspeex/sb_celp.c	(working copy)
@@ -46,15 +46,24 @@
 #include "ltp.h"
 #include "misc.h"
 
+
 /* Default size for the encoder and decoder stack (can be changed at compile time).
    This does not apply when using variable-size arrays or alloca. */
 #ifndef SB_ENC_STACK
-#define SB_ENC_STACK (10000*sizeof(spx_sig_t))
+#ifdef _USE_SSE
+#define SB_ENC_STACK (14000*sizeof(spx_sig_t))
+#else
+#define SB_ENC_STACK (11000*sizeof(spx_sig_t))
 #endif
+#endif
 
 #ifndef SB_DEC_STACK
-#define SB_DEC_STACK (6000*sizeof(spx_sig_t))
+#ifdef _USE_SSE
+#define SB_DEC_STACK (10000*sizeof(spx_sig_t))
+#else
+#define SB_DEC_STACK (7000*sizeof(spx_sig_t))
 #endif
+#endif
 
 
 #ifdef DISABLE_WIDEBAND
@@ -803,7 +812,6 @@
       filter_mem2(sp, st->bw_lpc1, st->bw_lpc2, sw, st->subframeSize, st->lpcSize, st->mem_sw);
    }
 
-
 #ifdef RESYNTH
    /* Reconstruct the original */
    fir_mem_up(st->x0d, h0, st->y0, st->full_frame_size, QMF_ORDER, st->g0_mem, stack);
Index: libspeex/speex.c
===================================================================
--- libspeex/speex.c	(revision 10524)
+++ libspeex/speex.c	(working copy)
@@ -39,16 +39,38 @@
 #include "modes.h"
 #include <math.h>
 
+#include "filters.h"
+#include "ltp.h"
+
+#ifdef _USE_SSE
+#include "filters_sse.h"
+#include "cb_search_sse.h"
+#include "ltp_sse.h"
+#include "vq_sse.h"
+#elif defined _USE_ALTIVEC
+#include "filters_altivec.h"
+#include "ltp_altivec.h"
+#elif defined (ARM4_ASM) || defined(ARM5E_ASM)
+#include "filters_arm4.h"
+#include "cb_search_arm4.h"
+#include "vq_arm4.h"
+#include "ltp_arm4.h"
+#elif defined (BFIN_ASM)
+#include "filters_bfin.h"
+#include "cb_search_bfin.h"
+#include "vq_bfin.h"
+#include "ltp_bfin.h"
+#endif
+
 #ifndef NULL
 #define NULL 0
 #endif
 
 #define MAX_IN_SAMPLES 640
 
-
-
 void *speex_encoder_init(const SpeexMode *mode)
 {
+   speex_set_asm(0);
    return mode->enc_init(mode);
 }
 
@@ -59,7 +81,21 @@
 
 void speex_encoder_destroy(void *state)
 {
-   (*((SpeexMode**)state))->enc_destroy(state);
+#ifdef _USE_SSE
+        fptr_fm = 0;
+        fptr_iir = 0;
+        fptr_fir = 0;
+        fptr_ip = 0;
+        fptr_px = 0;
+        fptr_vnb = 0;
+        fptr_vnbs = 0;
+        fptr_cwc = 0;
+#elif defined _USE_ALTIVEC
+        fptr_fm = 0;
+        fptr_ip = 0;
+#endif
+
+	(*((SpeexMode**)state))->enc_destroy(state);
 }
 
 void speex_decoder_destroy(void *state)
@@ -68,7 +104,6 @@
 }
 
 
-
 int speex_encode_native(void *state, spx_word16_t *in, SpeexBits *bits)
 {
    return (*((SpeexMode**)state))->enc(state, in, bits);
@@ -170,20 +205,54 @@
 }
 #endif
 
-
-
-int speex_encoder_ctl(void *state, int request, void *ptr)
+void speex_set_asm(int use_asm)
 {
-   return (*((SpeexMode**)state))->enc_ctl(state, request, ptr);
+    if (use_asm)
+    {
+#ifdef _USE_SSE
+        fptr_fm = filter_mem2_sse;
+        fptr_iir = iir_mem2_sse;
+        fptr_fir = fir_mem2_sse;
+        fptr_ip = inner_prod_sse;
+        fptr_px = pitch_xcorr_sse;
+        fptr_vnb = vq_nbest_sse;
+        fptr_vnbs = vq_nbest_sign_sse;
+        fptr_cwc = compute_weighted_codebook_sse;
+#elif defined _USE_ALTIVEC
+        fptr_fm = filter_mem2_altivec;
+        fptr_ip = inner_prod_altivec;
+#endif
+    }
+    else
+    {
+#ifdef _USE_SSE
+	fptr_fm = filter_mem2_c;
+	fptr_iir = iir_mem2_c;
+	fptr_fir = fir_mem2_c;
+	fptr_ip = inner_prod_c;
+	fptr_px = pitch_xcorr_c;
+    fptr_vnb = vq_nbest_c;
+    fptr_vnbs = vq_nbest_sign_c;
+    fptr_cwc = compute_weighted_codebook_c;
+#elif defined _USE_ALTIVEC
+    fptr_fm = filter_mem2_c;
+    fptr_ip = inner_prod_c;
+#endif
+
+    }
+    return;
 }
 
-int speex_decoder_ctl(void *state, int request, void *ptr)
-{
-   return (*((SpeexMode**)state))->dec_ctl(state, request, ptr);
+int speex_encoder_ctl(void *state, int request, void *ptr)
+{
+    return (*((SpeexMode**)state))->enc_ctl(state, request, ptr);
+}
+
+int speex_decoder_ctl(void *state, int request, void *ptr)
+{
+    return (*((SpeexMode**)state))->dec_ctl(state, request, ptr);
 }
 
-
-
 int nb_mode_query(const void *mode, int request, void *ptr)
 {
    const SpeexNBMode *m = (const SpeexNBMode*)mode;
Index: libspeex/vbr.c
===================================================================
--- libspeex/vbr.c	(revision 10524)
+++ libspeex/vbr.c	(working copy)
@@ -119,6 +119,7 @@
   non-stationary (harder to notice high-frequency noise)???
 
 */
+
 
 float vbr_analysis(VBRState *vbr, spx_word16_t *sig, int len, int pitch, float pitch_coef)
 {
@@ -215,6 +216,15 @@
    }
    vbr->last_energy = ener;
    vbr->soft_pitch = .6*vbr->soft_pitch + .4*pitch_coef;
+
+    // this can be caused by ol_pitch going to NAN as
+    // a result of the gain calculation going awry in 
+    // open_loop_nbest_pitch()- look for the word "wonky"
+    if (!finite(vbr->soft_pitch))
+    {
+         vbr->soft_pitch = 0;
+    }
+
    qual += 2.2*((pitch_coef-.4) + (vbr->soft_pitch-.4));
 
    if (qual < vbr->last_quality)
Index: libspeex/vq.c
===================================================================
--- libspeex/vq.c	(revision 10524)
+++ libspeex/vq.c	(working copy)
@@ -38,16 +38,6 @@
 #include "stack_alloc.h"
 #include "misc.h"
 
-#ifdef _USE_SSE
-#include <xmmintrin.h>
-#include "vq_sse.h"
-#elif defined(SHORTCUTS) && (defined(ARM4_ASM) || defined(ARM5E_ASM))
-#include "vq_arm4.h"
-#elif defined(BFIN_ASM)
-#include "vq_bfin.h"
-#endif
-
-
 int scal_quant(spx_word16_t in, const spx_word16_t *boundary, int entries)
 {
    int i=0;
@@ -94,21 +84,23 @@
 }
 
 
-#ifndef OVERRIDE_VQ_NBEST
 /*Finds the indices of the n-best entries in a codebook*/
-void vq_nbest(spx_word16_t *in, const spx_word16_t *codebook, int len, int entries, spx_word32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
+#ifndef OVERRIDE_VQ_NBEST
+void vq_nbest_c(spx_word16_t *in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
 {
    int i,j,k,used;
+   int pos = 0;
    used = 0;
+
    for (i=0;i<entries;i++)
    {
       spx_word32_t dist=0;
       for (j=0;j<len;j++)
-         dist = MAC16_16(dist,in[j],*codebook++);
+         dist = MAC16_16(dist,in[j],codebook[pos++].f[0]);
 #ifdef FIXED_POINT
-      dist=SUB32(SHR32(E[i],1),dist);
+      dist=SUB32(SHR32(E[i].f[0],1),dist);
 #else
-      dist=.5f*E[i]-dist;
+      dist=.5f*E[i].f[0]-dist;
 #endif
       if (i<N || dist<best_dist[N-1])
       {
@@ -125,20 +117,17 @@
 }
 #endif
 
-
-
-
-#ifndef OVERRIDE_VQ_NBEST_SIGN
 /*Finds the indices of the n-best entries in a codebook with sign*/
-void vq_nbest_sign(spx_word16_t *in, const spx_word16_t *codebook, int len, int entries, spx_word32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
+void vq_nbest_sign_c(spx_word16_t *in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
 {
    int i,j,k, sign, used;
+   int pos = 0;
    used=0;
    for (i=0;i<entries;i++)
    {
       spx_word32_t dist=0;
       for (j=0;j<len;j++)
-         dist = MAC16_16(dist,in[j],*codebook++);
+         dist = MAC16_16(dist,in[j],codebook[pos++].f[0]);
       if (dist>0)
       {
          sign=0;
@@ -148,9 +137,9 @@
          sign=1;
       }
 #ifdef FIXED_POINT
-      dist = ADD32(dist,SHR32(E[i],1));
+      dist = ADD32(dist,SHR32(E[i].f[0],1));
 #else
-      dist = ADD32(dist,.5f*E[i]);
+      dist = ADD32(dist,.5f*E[i].f[0]);
 #endif
       if (i<N || dist<best_dist[N-1])
       {
@@ -167,4 +156,3 @@
       }
    }
 }
-#endif
Index: libspeex/vq.h
===================================================================
--- libspeex/vq.h	(revision 10524)
+++ libspeex/vq.h	(working copy)
@@ -39,17 +39,6 @@
 
 int scal_quant(spx_word16_t in, const spx_word16_t *boundary, int entries);
 int scal_quant32(spx_word32_t in, const spx_word32_t *boundary, int entries);
-
 int vq_index(float *in, const float *codebook, int len, int entries);
-#ifdef _USE_SSE
-#include <xmmintrin.h>
-void vq_nbest(spx_word16_t *in, const __m128 *codebook, int len, int entries, __m128 *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
 
-void vq_nbest_sign(spx_word16_t *in, const __m128 *codebook, int len, int entries, __m128 *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
-#else
-void vq_nbest(spx_word16_t *in, const spx_word16_t *codebook, int len, int entries, spx_word32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
-
-void vq_nbest_sign(spx_word16_t *in, const spx_word16_t *codebook, int len, int entries, spx_word32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
 #endif
-
-#endif
Index: libspeex/vq_sse.h
===================================================================
--- libspeex/vq_sse.h	(revision 10524)
+++ libspeex/vq_sse.h	(working copy)
@@ -32,89 +32,15 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#define OVERRIDE_VQ_NBEST
-void vq_nbest(spx_word16_t *_in, const __m128 *codebook, int len, int entries, __m128 *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
-{
-   int i,j,k,used;
-   VARDECL(float *dist);
-   VARDECL(__m128 *in);
-   __m128 half;
-   used = 0;
-   ALLOC(dist, entries, float);
-   half = _mm_set_ps1(.5f);
-   ALLOC(in, len, __m128);
-   for (i=0;i<len;i++)
-      in[i] = _mm_set_ps1(_in[i]);
-   for (i=0;i<entries>>2;i++)
-   {
-      __m128 d = _mm_mul_ps(E[i], half);
-      for (j=0;j<len;j++)
-         d = _mm_sub_ps(d, _mm_mul_ps(in[j], *codebook++));
-      _mm_storeu_ps(dist+4*i, d);
-   }
-   for (i=0;i<entries;i++)
-   {
-      if (i<N || dist[i]<best_dist[N-1])
-      {
-         for (k=N-1; (k >= 1) && (k > used || dist[i] < best_dist[k-1]); k--)
-         {
-            best_dist[k]=best_dist[k-1];
-            nbest[k] = nbest[k-1];
-         }
-         best_dist[k]=dist[i];
-         nbest[k]=i;
-         used++;
-      }
-   }
-}
 
+#ifndef _VQNBEST_SSE_
+#define _VQNBEST_SSE_
 
+#include <xmmintrin.h>
+#include "stack_alloc.h"
+#include "misc.h"
 
+void vq_nbest_sse(spx_word16_t *_in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
+void vq_nbest_sign_sse(spx_word16_t *_in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
 
-#define OVERRIDE_VQ_NBEST_SIGN
-void vq_nbest_sign(spx_word16_t *_in, const __m128 *codebook, int len, int entries, __m128 *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
-{
-   int i,j,k,used;
-   VARDECL(float *dist);
-   VARDECL(__m128 *in);
-   __m128 half;
-   used = 0;
-   ALLOC(dist, entries, float);
-   half = _mm_set_ps1(.5f);
-   ALLOC(in, len, __m128);
-   for (i=0;i<len;i++)
-      in[i] = _mm_set_ps1(_in[i]);
-   for (i=0;i<entries>>2;i++)
-   {
-      __m128 d = _mm_setzero_ps();
-      for (j=0;j<len;j++)
-         d = _mm_add_ps(d, _mm_mul_ps(in[j], *codebook++));
-      _mm_storeu_ps(dist+4*i, d);
-   }
-   for (i=0;i<entries;i++)
-   {
-      int sign;
-      if (dist[i]>0)
-      {
-         sign=0;
-         dist[i]=-dist[i];
-      } else
-      {
-         sign=1;
-      }
-      dist[i] += .5f*((float*)E)[i];
-      if (i<N || dist[i]<best_dist[N-1])
-      {
-         for (k=N-1; (k >= 1) && (k > used || dist[i] < best_dist[k-1]); k--)
-         {
-            best_dist[k]=best_dist[k-1];
-            nbest[k] = nbest[k-1];
-         }
-         best_dist[k]=dist[i];
-         nbest[k]=i;
-         used++;
-         if (sign)
-            nbest[k]+=entries;
-      }
-   }
-}
+#endif
Index: macosx/Speex.xcodeproj/project.pbxproj
===================================================================
--- macosx/Speex.xcodeproj/project.pbxproj	(revision 10524)
+++ macosx/Speex.xcodeproj/project.pbxproj	(working copy)
@@ -7,6 +7,10 @@
 	objects = {
 
 /* Begin PBXBuildFile section */
+		1905A06F093CF172001F65AA /* filters_altivec.c in Sources */ = {isa = PBXBuildFile; fileRef = 1905A06D093CF172001F65AA /* filters_altivec.c */; };
+		1905A070093CF172001F65AA /* filters_altivec.h in Headers */ = {isa = PBXBuildFile; fileRef = 1905A06E093CF172001F65AA /* filters_altivec.h */; };
+		1905A073093CF18B001F65AA /* ltp_altivec.c in Sources */ = {isa = PBXBuildFile; fileRef = 1905A071093CF18B001F65AA /* ltp_altivec.c */; };
+		1905A074093CF18B001F65AA /* ltp_altivec.h in Headers */ = {isa = PBXBuildFile; fileRef = 1905A072093CF18B001F65AA /* ltp_altivec.h */; };
 		73814AFF0907FB8200C478FC /* speex_echo.h in Headers */ = {isa = PBXBuildFile; fileRef = 73814AF30907FB8200C478FC /* speex_echo.h */; settings = {ATTRIBUTES = (Public, ); }; };
 		73814B000907FB8200C478FC /* speex_header.h in Headers */ = {isa = PBXBuildFile; fileRef = 73814AF40907FB8200C478FC /* speex_header.h */; settings = {ATTRIBUTES = (Public, ); }; };
 		73814B010907FB8200C478FC /* speex_jitter.h in Headers */ = {isa = PBXBuildFile; fileRef = 73814AF50907FB8200C478FC /* speex_jitter.h */; settings = {ATTRIBUTES = (Public, ); }; };
@@ -79,6 +83,10 @@
 
 /* Begin PBXFileReference section */
 		089C1667FE841158C02AAC07 /* English */ = {isa = PBXFileReference; fileEncoding = 10; lastKnownFileType = text.plist.strings; name = English; path = English.lproj/InfoPlist.strings; sourceTree = "<group>"; };
+		1905A06D093CF172001F65AA /* filters_altivec.c */ = {isa = PBXFileReference; fileEncoding = 30; lastKnownFileType = sourcecode.c.c; name = filters_altivec.c; path = ../libspeex/filters_altivec.c; sourceTree = SOURCE_ROOT; };
+		1905A06E093CF172001F65AA /* filters_altivec.h */ = {isa = PBXFileReference; fileEncoding = 30; lastKnownFileType = sourcecode.c.h; name = filters_altivec.h; path = ../libspeex/filters_altivec.h; sourceTree = SOURCE_ROOT; };
+		1905A071093CF18B001F65AA /* ltp_altivec.c */ = {isa = PBXFileReference; fileEncoding = 30; lastKnownFileType = sourcecode.c.c; name = ltp_altivec.c; path = ../libspeex/ltp_altivec.c; sourceTree = SOURCE_ROOT; };
+		1905A072093CF18B001F65AA /* ltp_altivec.h */ = {isa = PBXFileReference; fileEncoding = 30; lastKnownFileType = sourcecode.c.h; name = ltp_altivec.h; path = ../libspeex/ltp_altivec.h; sourceTree = SOURCE_ROOT; };
 		32BAE0B70371A74B00C91783 /* Speex_Prefix.pch */ = {isa = PBXFileReference; fileEncoding = 4; lastKnownFileType = sourcecode.c.h; path = Speex_Prefix.pch; sourceTree = "<group>"; };
 		73814AEF0907FB8200C478FC /* speex.h */ = {isa = PBXFileReference; fileEncoding = 30; lastKnownFileType = sourcecode.c.h; path = speex.h; sourceTree = "<group>"; };
 		73814AF00907FB8200C478FC /* speex_bits.h */ = {isa = PBXFileReference; fileEncoding = 30; lastKnownFileType = sourcecode.c.h; path = speex_bits.h; sourceTree = "<group>"; };
@@ -231,6 +239,8 @@
 				73814B160907FC9900C478FC /* exc_10_16_table.c */,
 				73814B170907FC9900C478FC /* exc_10_32_table.c */,
 				73814B180907FC9900C478FC /* exc_20_32_table.c */,
+				1905A06D093CF172001F65AA /* filters_altivec.c */,
+				1905A06E093CF172001F65AA /* filters_altivec.h */,
 				73814B190907FC9900C478FC /* filters_arm4.h */,
 				73814B1A0907FC9900C478FC /* filters_bfin.h */,
 				73814B1B0907FC9900C478FC /* filters_sse.h */,
@@ -257,6 +267,8 @@
 				73814B300907FC9900C478FC /* ltp_arm4.h */,
 				73814B310907FC9900C478FC /* ltp_bfin.h */,
 				73814B320907FC9900C478FC /* ltp_sse.h */,
+				1905A071093CF18B001F65AA /* ltp_altivec.c */,
+				1905A072093CF18B001F65AA /* ltp_altivec.h */,
 				73814B330907FC9900C478FC /* ltp.c */,
 				73814B340907FC9900C478FC /* ltp.h */,
 				73814B370907FC9900C478FC /* math_approx.c */,
@@ -339,6 +351,8 @@
 				73814B060907FBAB00C478FC /* speex_callbacks.h in Headers */,
 				73814B070907FBAD00C478FC /* speex_bits.h in Headers */,
 				73814B080907FBAE00C478FC /* speex.h in Headers */,
+				1905A070093CF172001F65AA /* filters_altivec.h in Headers */,
+				1905A074093CF18B001F65AA /* ltp_altivec.h in Headers */,
 			);
 			runOnlyForDeploymentPostprocessing = 0;
 		};
@@ -460,6 +474,8 @@
 				73814B990907FC9900C478FC /* stereo.c in Sources */,
 				73814B9F0907FC9900C478FC /* vbr.c in Sources */,
 				73814BA40907FC9900C478FC /* vq.c in Sources */,
+				1905A06F093CF172001F65AA /* filters_altivec.c in Sources */,
+				1905A073093CF18B001F65AA /* ltp_altivec.c in Sources */,
 			);
 			runOnlyForDeploymentPostprocessing = 0;
 		};
@@ -542,21 +558,33 @@
 		73814AE40907FB1E00C478FC /* Development */ = {
 			isa = XCBuildConfiguration;
 			buildSettings = {
-				GCC_PREPROCESSOR_DEFINITIONS = __MACOSX__;
+				GCC_ALTIVEC_EXTENSIONS = YES;
+				GCC_PREPROCESSOR_DEFINITIONS = (
+					__MACOSX__,
+					_USE_ALTIVEC,
+				);
 			};
 			name = Development;
 		};
 		73814AE50907FB1E00C478FC /* Deployment */ = {
 			isa = XCBuildConfiguration;
 			buildSettings = {
-				GCC_PREPROCESSOR_DEFINITIONS = __MACOSX__;
+				GCC_ALTIVEC_EXTENSIONS = YES;
+				GCC_PREPROCESSOR_DEFINITIONS = (
+					__MACOSX__,
+					_USE_ALTIVEC,
+				);
 			};
 			name = Deployment;
 		};
 		73814AE60907FB1E00C478FC /* Default */ = {
 			isa = XCBuildConfiguration;
 			buildSettings = {
-				GCC_PREPROCESSOR_DEFINITIONS = __MACOSX__;
+				GCC_ALTIVEC_EXTENSIONS = YES;
+				GCC_PREPROCESSOR_DEFINITIONS = (
+					__MACOSX__,
+					_USE_ALTIVEC,
+				);
 			};
 			name = Default;
 		};
Index: win32/config.h
===================================================================
--- win32/config.h	(revision 10524)
+++ win32/config.h	(working copy)
@@ -1,3 +1,5 @@
 #define inline __inline
 #define restrict
+#include <float.h>
+#define finite _finite
 #include "misc.h"
\ No newline at end of file
Index: win32/VS2003/libspeex/libspeex.vcproj
===================================================================
--- win32/VS2003/libspeex/libspeex.vcproj	(revision 10524)
+++ win32/VS2003/libspeex/libspeex.vcproj	(working copy)
@@ -19,11 +19,12 @@
 			<Tool
 				Name="VCCLCompilerTool"
 				Optimization="0"
+				EnableIntrinsicFunctions="TRUE"
 				AdditionalIncludeDirectories="..\..\..\include;..\.."
-				PreprocessorDefinitions="WIN32;_DEBUG;_LIB;HAVE_CONFIG_H"
+				PreprocessorDefinitions="WIN32;_DEBUG;_LIB;HAVE_CONFIG_H;_USE_SSE"
 				MinimalRebuild="TRUE"
 				BasicRuntimeChecks="3"
-				RuntimeLibrary="1"
+				RuntimeLibrary="3"
 				UsePrecompiledHeader="0"
 				WarningLevel="4"
 				Detect64BitPortabilityProblems="TRUE"
@@ -33,7 +34,7 @@
 				Name="VCCustomBuildTool"/>
 			<Tool
 				Name="VCLibrarianTool"
-				OutputFile="$(OutDir)/libspeex.lib"/>
+				OutputFile="../../Output/lib/libspeex-Static-Debug-MTDLL.lib"/>
 			<Tool
 				Name="VCMIDLTool"/>
 			<Tool
@@ -68,11 +69,12 @@
 				EnableIntrinsicFunctions="TRUE"
 				FavorSizeOrSpeed="1"
 				OptimizeForProcessor="2"
+				OptimizeForWindowsApplication="TRUE"
 				AdditionalIncludeDirectories="..\..\..\include;..\.."
-				PreprocessorDefinitions="WIN32;NDEBUG;_LIB;HAVE_CONFIG_H"
+				PreprocessorDefinitions="WIN32;NDEBUG;_LIB;HAVE_CONFIG_H;_USE_SSE"
 				StringPooling="TRUE"
 				ExceptionHandling="FALSE"
-				RuntimeLibrary="0"
+				RuntimeLibrary="2"
 				BufferSecurityCheck="FALSE"
 				UsePrecompiledHeader="0"
 				WarningLevel="4"
@@ -84,7 +86,7 @@
 				Name="VCCustomBuildTool"/>
 			<Tool
 				Name="VCLibrarianTool"
-				OutputFile="$(OutDir)/libspeex.lib"/>
+				OutputFile="../../Output/lib/libspeex-Static-MTDLL.lib"/>
 			<Tool
 				Name="VCMIDLTool"/>
 			<Tool
@@ -172,7 +174,7 @@
 				FavorSizeOrSpeed="1"
 				OptimizeForProcessor="3"
 				AdditionalIncludeDirectories="..\..\..\include;..\.."
-				PreprocessorDefinitions="WIN32;NDEBUG;_LIB;HAVE_CONFIG_H"
+				PreprocessorDefinitions="WIN32;NDEBUG;_LIB;HAVE_CONFIG_H;USE_SSE"
 				StringPooling="TRUE"
 				ExceptionHandling="FALSE"
 				RuntimeLibrary="0"
@@ -279,6 +281,9 @@
 				RelativePath="..\..\..\libspeex\cb_search.c">
 			</File>
 			<File
+				RelativePath="..\..\..\libspeex\cb_search_sse.c">
+			</File>
+			<File
 				RelativePath="..\..\..\libspeex\exc_10_16_table.c">
 			</File>
 			<File
@@ -297,9 +302,15 @@
 				RelativePath="..\..\..\libspeex\exc_8_128_table.c">
 			</File>
 			<File
+				RelativePath="..\..\..\libspeex\fftwrap.c">
+			</File>
+			<File
 				RelativePath="..\..\..\libspeex\filters.c">
 			</File>
 			<File
+				RelativePath="..\..\..\libspeex\filters_sse.c">
+			</File>
+			<File
 				RelativePath="..\..\..\libspeex\gain_table.c">
 			</File>
 			<File
@@ -318,6 +329,12 @@
 				RelativePath="..\..\..\libspeex\jitter.c">
 			</File>
 			<File
+				RelativePath="..\..\..\libspeex\kiss_fft.c">
+			</File>
+			<File
+				RelativePath="..\..\..\libspeex\kiss_fftr.c">
+			</File>
+			<File
 				RelativePath=".\libspeex.def">
 			</File>
 			<File
@@ -333,6 +350,9 @@
 				RelativePath="..\..\..\libspeex\ltp.c">
 			</File>
 			<File
+				RelativePath="..\..\..\libspeex\ltp_sse.c">
+			</File>
+			<File
 				RelativePath="..\..\..\libspeex\math_approx.c">
 			</File>
 			<File
@@ -380,24 +400,42 @@
 			<File
 				RelativePath="..\..\..\libspeex\vq.c">
 			</File>
+			<File
+				RelativePath="..\..\..\libspeex\vq_sse.c">
+			</File>
 		</Filter>
 		<Filter
 			Name="Header Files"
 			Filter="h;hpp;hxx;hm;inl;inc;xsd"
 			UniqueIdentifier="{93995380-89BD-4b04-88EB-625FBE52EBFB}">
 			<File
+				RelativePath="..\..\..\libspeex\arch.h">
+			</File>
+			<File
 				RelativePath="..\..\..\libspeex\cb_search.h">
 			</File>
 			<File
+				RelativePath="..\..\..\libspeex\cb_search_sse.h">
+			</File>
+			<File
 				RelativePath="..\..\config.h">
 			</File>
 			<File
+				RelativePath="..\..\..\libspeex\fftwrap.h">
+			</File>
+			<File
 				RelativePath="..\..\..\libspeex\filters.h">
 			</File>
 			<File
 				RelativePath="..\..\..\libspeex\filters_sse.h">
 			</File>
 			<File
+				RelativePath="..\..\..\libspeex\kiss_fft.h">
+			</File>
+			<File
+				RelativePath="..\..\..\libspeex\kiss_fftr.h">
+			</File>
+			<File
 				RelativePath="..\..\..\libspeex\lpc.h">
 			</File>
 			<File
@@ -466,6 +504,9 @@
 			<File
 				RelativePath="..\..\..\libspeex\vq.h">
 			</File>
+			<File
+				RelativePath="..\..\..\libspeex\vq_sse.h">
+			</File>
 		</Filter>
 		<Filter
 			Name="Resource Files"